package com.neurosdk2;

import android.util.Log;

import androidx.annotation.NonNull;

import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.neurosdk2.neuro.ISensor;
import com.neurosdk2.neuro.Scanner;
import com.neurosdk2.neuro.Sensor;

import com.neurosdk2.neuro.BrainBit2;


import com.neurosdk2.neuro.NeuroEEG;

import com.neurosdk2.neuro.types.SensorFamily;
import com.neurosdk2.neuro.types.SensorInfo;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class Neurosdk2Module extends com.neurosdk2.Neurosdk2Spec {
  public static final String NAME = "Neurosdk2";


  private final ExecutorService _threadService;
  ScannerHolder scannerHolder;
  SensorHolder sensorHolder;

  Neurosdk2Module(ReactApplicationContext context) {
    super(context);
    scannerHolder = new ScannerHolder(context);
    sensorHolder = new SensorHolder(context);
    _threadService = Executors.newCachedThreadPool();
  }

  @Override
  @NonNull
  public String getName() {
    return NAME;
  }

  // <editor-fold> ScannerImpl
  @ReactMethod
  @Override
  public void createScanner(ReadableArray filters, Promise promise) {
    scannerHolder.createScanner(filters, promise);
  }

  @ReactMethod
  @Override
  public void closeScanner(String guid, Promise promise) {
    scannerHolder.closeScanner(guid, promise);
  }

  @ReactMethod
  @Override
  public void startScan(String guid, Promise promise) {
    scannerHolder.startScan(guid, promise);
  }

  @ReactMethod
  @Override
  public void stopScan(String guid, Promise promise) {
    scannerHolder.stopScan(guid, promise);
  }

  @ReactMethod
  @Override
  public synchronized void createSensor(String guid, ReadableMap sensorInfo, Promise promise) {
    try {
      if(sensorInfo == null) promise.reject("createSensor", "Sensor info is null");

      SensorInfo info = new SensorInfo(
        SensorFamily.indexOf(sensorInfo.getInt("SensFamily")),
        (byte) sensorInfo.getInt("SensModel"),
        sensorInfo.getString("Name"),
        sensorInfo.getString("Address"),
        sensorInfo.getString("SerialNumber"),
        sensorInfo.getBoolean("PairingRequired"),
        (short) sensorInfo.getInt("RSSI"));

      _threadService.submit(() -> {
        try {
          Scanner scanner = scannerHolder.getScanner(guid);
          if(scanner == null)
          {
            promise.reject("createSensor", "Scanner don't exist");
            return;
          }

          String uuid = UUID.randomUUID().toString();
          Sensor sensor = scanner.createSensor(info);
          if(sensor == null)
          {
            promise.reject("createSensor", "Cannot create sensor");
            return;
          }
          sensorHolder.addSensor(uuid, sensor);
          promise.resolve(uuid);
        }
        catch (Exception e)
        {
          promise.reject("createSensor", e.getLocalizedMessage());
        }
      });

    } catch (Exception e) {
      promise.reject("createSensor", e.getLocalizedMessage());
    }
  }

  @ReactMethod
  @Override
  public void getSensors(String guid, Promise promise) {
    scannerHolder.getSensors(guid, promise);
  }
  // </editor-fold>

  // <editor-fold> Sensor
  @ReactMethod
  @Override
  public void closeSensor(String guid) {
    sensorHolder.closeSensor(guid);
  }

  @ReactMethod
  @Override
  public void connectSensor(String guid, Promise promise) {
    _threadService.submit(()->{
      sensorHolder.connectSensor(guid, promise);
    });
  }

  @ReactMethod
  @Override
  public void disconnectSensor(String guid, Promise promise) {
    sensorHolder.disconnectSensor(guid, promise);
  }
  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public WritableArray supportedFeatures(String guid) {
    return sensorHolder.supportedFeatures(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public boolean isSupportedFeature(String guid, double feature) {
    return sensorHolder.isSupportedFeature(guid, feature);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public WritableArray supportedCommands(String guid) {
    return sensorHolder.supportedCommands(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public boolean isSupportedCommand(String guid, double command) {
    return sensorHolder.isSupportedCommand(guid, command);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public WritableArray supportedParameters(String guid) {
    return sensorHolder.supportedParameters(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public boolean isSupportedParameter(String guid, double parameter) {
    return sensorHolder.isSupportedParameter(guid, parameter);
  }

  @ReactMethod
  @Override
  public void execCommand(String guid, double command, Promise promise) {
    _threadService.submit(()-> sensorHolder.execCommand(guid, command, promise));
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public String getName(String guid) {
    return sensorHolder.getName(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void setName(String guid, String name) {
    sensorHolder.setName(guid, name);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getState(String guid) {
    return sensorHolder.getState(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public String getAddress(String guid) {
    return sensorHolder.getAddress(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public String getSerialNumber(String guid) {
    return sensorHolder.getSerialNumber(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void setSerialNumber(String guid, String sn) {
    sensorHolder.setSerialNumber(guid, sn);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getBattPower(String guid) {
    return sensorHolder.getBattPower(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getSamplingFrequency(String guid) {
    return sensorHolder.getSamplingFrequency(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getGain(String guid) {
    return sensorHolder.getGain(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getDataOffset(String guid) {
    return sensorHolder.getDataOffset(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getFirmwareMode(String guid) {
    return sensorHolder.getFirmwareMode(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public WritableMap getVersion(String guid) {
    return sensorHolder.getVersion(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getChannelsCount(String guid) {
    return sensorHolder.getChannelsCount(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getSensFamily(String guid) {
    return sensorHolder.getSensFamily(guid);
  }

  // </editor-fold>


  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getSamplingFrequencyFPG(String guid) {
    return sensorHolder.getSamplingFrequencyFPG(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getIrAmplitude(String guid) {
    return sensorHolder.getIrAmplitude(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void setIrAmplitude(String guid, double amp) {
    sensorHolder.setIrAmplitude(guid, amp);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getRedAmplitude(String guid) {
    return sensorHolder.getRedAmplitude(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void setRedAmplitude(String guid, double amp) {
    sensorHolder.setRedAmplitude(guid, amp);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void pingNeuroSmart(String guid, double marker, Promise promise) {
    sensorHolder.pingNeuroSmart(guid, marker, promise);
  }


  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getAmpMode(String guid) {
    return sensorHolder.getAmpMode(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getSamplingFrequencyResist(String guid) {
    return sensorHolder.getSamplingFrequencyResist(guid);
  }


  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public boolean isSupportedFilter(String guid, double filter) {
    return sensorHolder.isSupportedFilter(guid, filter);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public WritableArray getSupportedFilters(String guid) {
    return sensorHolder.getSupportedFilters(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public WritableArray getHardwareFilters(String guid) {
    return sensorHolder.getHardwareFilters(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void setHardwareFilters(String guid, ReadableArray filters) {
    sensorHolder.setHardwareFilters(guid, filters);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void setFirmwareMode(String guid, double mode) {
    sensorHolder.setFirmwareMode(guid, mode);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void setSamplingFrequency(String guid, double sf) {
    sensorHolder.setSamplingFrequency(guid, sf);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void setGain(String guid, double gain) {
    sensorHolder.setGain(guid, gain);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void setDataOffset(String guid, double offset) {
    sensorHolder.setDataOffset(guid, offset);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getExtSwInput(String guid) {
    return sensorHolder.getExtSwInput(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void setExtSwInput(String guid, double extSwInp) {
    sensorHolder.setExtSwInput(guid, extSwInp);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getADCInput(String guid) {
    return sensorHolder.getADCInput(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void setADCInput(String guid, double adcInp) {
    sensorHolder.setADCInput(guid, adcInp);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public WritableMap getStimulatorMAState(String guid) {
    return sensorHolder.getStimulatorMAState(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public WritableMap getStimulatorParam(String guid) {
    return sensorHolder.getStimulatorParam(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void setStimulatorParam(String guid, ReadableMap param) {
    sensorHolder.setStimulatorParam(guid, param);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public WritableMap getMotionAssistantParam(String guid) {
    return sensorHolder.getMotionAssistantParam(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void setMotionAssistantParam(String guid, ReadableMap param) {
    sensorHolder.setMotionAssistantParam(guid, param);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public WritableMap getMotionCounterParam(String guid) {
    return sensorHolder.getMotionCounterParam(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void setMotionCounterParam(String guid, ReadableMap param) {
    sensorHolder.setMotionCounterParam(guid, param);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getMotionCounter(String guid) {
    return sensorHolder.getMotionCounter(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getColor(String guid) {
    return sensorHolder.getColor(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public boolean getMEMSCalibrateState(String guid) {
    return sensorHolder.getMEMSCalibrateState(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getSamplingFrequencyResp(String guid) {
    return sensorHolder.getSamplingFrequencyResp(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getSamplingFrequencyEnvelope(String guid) {
    return sensorHolder.getSamplingFrequencyEnvelope(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getSignalType(String guid) {
    return sensorHolder.getSignalType(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void setSignalType(String guid, double type) {
    sensorHolder.setSignalType(guid, type);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getElectrodeState(String guid) {
    return sensorHolder.getElectrodeState(guid);
  }


  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getAccSens(String guid) {
    return sensorHolder.getAccSens(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void setAccSens(String guid, double accSens) {
    sensorHolder.setAccSens(guid, accSens);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getGyroSens(String guid) {
    return sensorHolder.getGyroSens(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void setGyroSens(String guid, double gyroSens) {
    sensorHolder.setGyroSens(guid, gyroSens);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getSamplingFrequencyMEMS(String guid) {
    return sensorHolder.getSamplingFrequencyMEMS(guid);
  }



  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double getSurveyId(String guid) {
    return sensorHolder.getSurveyId(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void setSurveyId(String guid, double id) {
    sensorHolder.setSurveyId(guid, id);
  }


  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public WritableMap getAmplifierParam(String guid) {
    return sensorHolder.getAmplifierParam(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public void setAmplifierParam(String guid, ReadableMap param) {
    sensorHolder.setAmplifierParam(guid, param);
  }


  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public WritableArray getSupportedChannels(String guid) {
    return sensorHolder.getSupportedChannels(guid);
  }


  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public WritableMap getAmplifierParamBB2(String guid) {
    return sensorHolder.getAmplifierParamBB2(guid);
  }

  @ReactMethod(isBlockingSynchronousMethod = true)
  @Override
  public double setAmplifierParamBB2(String guid, ReadableMap param) {
    return sensorHolder.setAmplifierParamBB2(guid, param);
  }




  @ReactMethod
  public void addListener(String eventName) {

  }

  @ReactMethod
  public void removeListeners(Integer count) {

  }

  @Override
  public Map<String, Object> getConstants() {

    final Map<String, Object> constants = new HashMap<>();
    // Scanner
    constants.put(Constants.SENSOR_LIST_CHANGED_EVENT_NAME.name(), Constants.SENSOR_LIST_CHANGED_EVENT_NAME.raw());
    // Sensor
    constants.put(Constants.CONNECTION_CHANGED_EVENT_NAME.name(),  Constants.CONNECTION_CHANGED_EVENT_NAME.raw());
    constants.put(Constants.BATTERY_CHANGED_EVENT_NAME.name(),  Constants.BATTERY_CHANGED_EVENT_NAME.raw());

    constants.put(Constants.BRAINBIT_SIGNAL_CHANGED_EVENT_NAME.name(),  Constants.BRAINBIT_SIGNAL_CHANGED_EVENT_NAME.raw());
    constants.put(Constants.BRAINBIT_RESISTANCE_CHANGED_EVENT_NAME.name(),  Constants.BRAINBIT_RESISTANCE_CHANGED_EVENT_NAME.raw());


    constants.put(Constants.AMPMODE_CHANGED_EVENT_NAME.name(),  Constants.AMPMODE_CHANGED_EVENT_NAME.raw());


    constants.put(Constants.NEUROSMART_FPGDATA_CHANGED_EVENT_NAME.name(),  Constants.NEUROSMART_FPGDATA_CHANGED_EVENT_NAME.raw());


    constants.put(Constants.CALLIBRI_ELECTRODE_STATE_EVENT_NAME.name(),  Constants.CALLIBRI_ELECTRODE_STATE_EVENT_NAME.raw());
    constants.put(Constants.CALLIBRI_ENVELOPE_DATA_EVENT_NAME.name(),  Constants.CALLIBRI_ENVELOPE_DATA_EVENT_NAME.raw());
    constants.put(Constants.CALLIBRI_SIGNAL_DATA_EVENT_NAME.name(),  Constants.CALLIBRI_SIGNAL_DATA_EVENT_NAME.raw());
    constants.put(Constants.CALLIBRI_RESPIRATION_DATA_EVENT_NAME.name(),  Constants.CALLIBRI_RESPIRATION_DATA_EVENT_NAME.raw());
    constants.put(Constants.QUATERNIONDATA_CHANGED_EVENT_NAME.name(),  Constants.QUATERNIONDATA_CHANGED_EVENT_NAME.raw());


    constants.put(Constants.MEMSDATA_CHANGED_EVENT_NAME.name(),  Constants.MEMSDATA_CHANGED_EVENT_NAME.raw());


    constants.put(Constants.NEURRO_EEG_SIGNAL_DATA_EVENT_NAME.name(),  Constants.NEURRO_EEG_SIGNAL_DATA_EVENT_NAME.raw());
    constants.put(Constants.NEURRO_EEG_RESIST_DATA_EVENT_NAME.name(),  Constants.NEURRO_EEG_RESIST_DATA_EVENT_NAME.raw());
    constants.put(Constants.NEURRO_EEG_SIGNAL_RESIST_DATA_EVENT_NAME.name(),  Constants.NEURRO_EEG_SIGNAL_RESIST_DATA_EVENT_NAME.raw());
    constants.put("NEURRO_EEG_MAX_CHANNELS_COUNT",  NeuroEEG.NEURO_EEG_MAX_CH_COUNT);


    constants.put(Constants.BRAINBIT2_SIGNAL_RECEIVED_EVENT_NAME.name(),  Constants.BRAINBIT2_SIGNAL_RECEIVED_EVENT_NAME.raw());
    constants.put(Constants.BRAINBIT2_RESIST_RECEIVED_EVENT_NAME.name(),  Constants.BRAINBIT2_RESIST_RECEIVED_EVENT_NAME.raw());
    constants.put("BRAINBIT2_MAX_CHANNELS_COUNT",  BrainBit2.BRAINBIT2_MAX_CH_COUNT);



    return constants;
  }
}
