//
//  ScannerHolder.m
//  Neurosdk2
//
//  Created by Aseatari on 02.05.2023.
//

#import <Foundation/Foundation.h>
#import "ScannerHolder.h"
#import "Neurosdk2.h"
#import "Constants.h"


@implementation ScannerHolder{
    NSMutableDictionary<NSString*, NTScanner*>* _scanners;
}

- (nonnull instancetype)init{
    self = [super init];
    if (self) {
        _scanners = [NSMutableDictionary new];
      }
    
    return self;
}

- (void) createScanner:(NSArray *_Nonnull)filters
               resolve:(RCTPromiseResolveBlock _Nonnull )resolve
                reject:(RCTPromiseRejectBlock _Nonnull )reject
             rncontext:(Neurosdk2* _Nonnull)rncontext{
    if(filters == nil){
        reject(@"createScanner", @"filters is empty", nil);
        return;
    }
    NSString *uuid = [[NSUUID UUID] UUIDString];
    NSLog(@"Create scanner: %@", uuid);
    @try {
        NTScanner* scanner = [[NTScanner alloc] initWithSensorFamily:filters];
        __weak ScannerHolder* weakSelf = self;
        [scanner setSensorsCallback:^(NSArray<NTSensorInfo*>* sensors){
            NSLog(@"Found %lu sensors", (unsigned long)[sensors count]);
            NSMutableArray* sensorsArray = [[NSMutableArray alloc] initWithCapacity: [sensors count]];
            for (NTSensorInfo* info in sensors) {
                NSLog(@"Found %@ [%@]", info.Name, info.SerialNumber);
                NSDictionary *sensor = @{ @"Name" : info.Name,
                                          @"Address" : info.Address,
                                          @"SerialNumber" : info.SerialNumber,
                                          @"SensModel": [NSNumber numberWithInt: info.SensModel],
                                          @"SensFamily": [NSNumber numberWithUnsignedInteger: info.SensFamily],
                                          @"PairingRequired": [NSNumber numberWithBool: info.PairingRequared],
                                          @"RSSI": [NSNumber numberWithShort:info.RSSI]
                };
                [sensorsArray addObject:sensor];
            }
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:uuid forKey:GUID_ID];
            [res setObject:sensorsArray forKey:DATA_ID];
            NSString* sensFoundedEvent = SENSOR_LIST_CHANGED_EVENT_NAME;
            [(Neurosdk2*)weakSelf.rncontext neuroSendEvent:sensFoundedEvent params:res];
        }];
        
        [_scanners setObject:scanner forKey:uuid];
        self.rncontext = rncontext;
        resolve(uuid);
    } @catch (NSException *exception) {
        reject(@"Error while scanner create: ", exception.description, nil);
    }
    
}

- (NTScanner*_Nullable) getScanner:(NSString*_Nonnull)guid{
    return _scanners[guid];
}

- (void) closeScanner:(NSString *_Nonnull)guid
             resolve:(RCTPromiseResolveBlock _Nonnull )resolve
               reject:(RCTPromiseRejectBlock _Nonnull )reject{
    @try {
        NSLog(@"Close scanner: %@", guid);
        NTScanner* scanner = [_scanners objectForKey:guid];
        [scanner setSensorsCallback:nil];
        scanner = nil;
        [_scanners removeObjectForKey:guid];
        resolve(@"success");
     }
     @catch (NSException * e) {
        NSLog(@"Exception: %@", e);
         reject(@"closeScanner", e.description, nil);
     }
    
}
//getScanner
- (void) startScan:(NSString *_Nonnull)guid
          resolve:(RCTPromiseResolveBlock _Nonnull )resolve
            reject:(RCTPromiseRejectBlock _Nonnull )reject{
    @try {
        NTScanner* scanner = [_scanners objectForKey:guid];
        if(scanner == nil) @throw [[NSException alloc] initWithName:@"NilScanner" reason:@"Scanner does not exist" userInfo:nil];
        [scanner startScan];
        NSLog(@"Scanner started, %@", guid);
            resolve(@"success");
    } @catch (NSException *e) {
        reject(@"startScan", e.description, nil);
    }
    
}

- (void) stopScan:(NSString *_Nonnull)guid
         resolve:(RCTPromiseResolveBlock _Nonnull )resolve
           reject:(RCTPromiseRejectBlock _Nonnull )reject{
    @try {
        NTScanner* scanner = [_scanners objectForKey:guid];
        if(scanner == nil) @throw [[NSException alloc] initWithName:@"NilScanner" reason:@"Scanner does not exist" userInfo:nil];
        [scanner stopScan];
        NSLog(@"Scanner stopped, %@", guid);
        resolve(@"success");
    } @catch (NSException *e) {
        reject(@"stopScan", e.description, nil);
    }
}

- (void) getSensors:(NSString *_Nonnull)guid
           resolve:(RCTPromiseResolveBlock _Nonnull )resolve
             reject:(RCTPromiseRejectBlock _Nonnull )reject{
    @try {
        NTScanner* scanner = [_scanners objectForKey:guid];
        if(scanner != nil){
            NSMutableArray<NSDictionary*>* result = [NSMutableArray new];
            NSArray<NTSensorInfo *> * sensors = [scanner sensors];
                for (NTSensorInfo* info in sensors){
                    NSDictionary *sensor = @{ @"Name" : info.Name,
                                              @"Address" : info.Address,
                                              @"SerialNumber" : info.SerialNumber,
                                              @"SensModel": [NSNumber numberWithInt: info.SensModel],
                                              @"SensFamily": [NSNumber numberWithUnsignedInteger: info.SensFamily],
                                              @"PairingRequired": [NSNumber numberWithBool: info.PairingRequared],
                                              @"RSSI": [NSNumber numberWithShort:info.RSSI]
                    };
                    [result addObject:sensor];
                }
            resolve(result);
        }else{
            @throw [[NSException alloc] initWithName:@"NilScanner" reason:@"Scanner does not exist" userInfo:nil];
        }
    } @catch (NSException *e) {
        reject(@"getSensors", e.description, nil);
    }
    
}


@end
