import { EmitterSubscription, NativeEventEmitter } from 'react-native';
import { Neurosdk2 } from "./ModuleResolver";

import type {
    SensorState,
    SensorSamplingFrequency,

    SensorCommand,
    ParameterInfo,
    SensorFeature,
    SensorFamily,
    SensorParameter,
    SensorVersion,
    SensorFirmwareMode,
    SensorDataOffset,
} from './NativeNeurosdk2'

export default class Sensor {
    protected guid: string
    protected nativeEventEmitter: NativeEventEmitter;

    private batteryChanged: EmitterSubscription | undefined;
    private connectionChanged: EmitterSubscription | undefined;

    constructor(guid: string) {
        this.nativeEventEmitter = new NativeEventEmitter(Neurosdk2);
        this.guid = guid;
    }

    close(): void {
        Neurosdk2.closeSensor(this.guid);
    }

    connect(): Promise<string> {
        return Neurosdk2.connectSensor(this.guid);
    }

    disconnect(): Promise<string> {
        return Neurosdk2.disconnectSensor(this.guid);
    }

    isSupportedFeature(feature: SensorFeature): boolean {
        return Neurosdk2.isSupportedFeature(this.guid, feature);
    }
    getFeatures(): Array<SensorFeature> {
        return Neurosdk2.supportedFeatures(this.guid);
    }

    isSupportedCommand(command: SensorCommand): boolean {
        return Neurosdk2.isSupportedCommand(this.guid, command);
    }
    getCommands(): Array<SensorCommand> {
        return Neurosdk2.supportedCommands(this.guid);
    }

    isSupportedParameter(parameters: SensorParameter): boolean {
        return Neurosdk2.isSupportedParameter(this.guid, parameters);
    }
    getParameters(): Array<ParameterInfo> {
        return Neurosdk2.supportedParameters(this.guid);
    }

    execute(command: SensorCommand): Promise<string> {
        return Neurosdk2.execCommand(this.guid, command);
    }

    getName():string{
        return Neurosdk2.getName(this.guid);
    }
    setName(value: string) {
        Neurosdk2.setName(this.guid, value);
    }

    getState(): SensorState {
        return Neurosdk2.getState(this.guid);
    }

    getAddress(): string {
        return Neurosdk2.getAddress(this.guid);
    }

    getSerialNumber(): string {
        return Neurosdk2.getSerialNumber(this.guid);
    }
    setSerialNumber(value: string) {
        Neurosdk2.setSerialNumber(this.guid, value);
    }

    getBattPower(): number {
        return Neurosdk2.getBattPower(this.guid);
    }

    getSamplingFrequency(): SensorSamplingFrequency {
        return Neurosdk2.getSamplingFrequency(this.guid);
    }

    getDataOffset(): SensorDataOffset {
        return Neurosdk2.getDataOffset(this.guid);
    }
    getFirmwareMode(): SensorFirmwareMode {
        return Neurosdk2.getFirmwareMode(this.guid);
    }
    getVersion(): SensorVersion {
        return Neurosdk2.getVersion(this.guid);
    }
    getChannelsCount(): number {
        return Neurosdk2.getChannelsCount(this.guid);
    }
    getSensFamily(): SensorFamily {
        return Neurosdk2.getSensFamily(this.guid);
    }
    //-----Callbacks-----//
    AddBatteryChanged(callback: (power: number) => void) {
        this.batteryChanged = this.nativeEventEmitter.addListener(
            Neurosdk2.BATTERY_CHANGED_EVENT_NAME,
            (p) => {
                if (p.GUID === this.guid) callback(p.data.power)
            }
        );
    }
    RemoveBatteryChanged() {
        if (this.batteryChanged != undefined)
            this.batteryChanged.remove();
    }
    AddConnectionChanged(callback: (state: SensorState) => void) {
        this.connectionChanged = this.nativeEventEmitter.addListener(
            Neurosdk2.CONNECTION_CHANGED_EVENT_NAME,
            (s) => { if (s.GUID === this.guid) callback(s.data.state) }
        );
    }
    RemoveConnectionChanged() {
        if (this.connectionChanged != undefined)
            this.connectionChanged.remove();
    }
}